﻿using Microsoft.EntityFrameworkCore.Internal;
using NetinaShop.Domain.Dtos.LargDtos;
using NetinaShop.Domain.Dtos.SmallDtos;

namespace NetinaShop.Repository.Handlers.Products;

public class GetProductsQueryHandler : IRequestHandler<GetProductsQuery, List<ProductSDto>>
{
    private readonly IRepositoryWrapper _repositoryWrapper;
    private readonly IMediator _mediator;

    public GetProductsQueryHandler(IRepositoryWrapper repositoryWrapper, IMediator mediator)
    {
        _repositoryWrapper = repositoryWrapper;
        _mediator = mediator;
    }
    public async Task<List<ProductSDto>> Handle(GetProductsQuery request, CancellationToken cancellationToken)
    {
        var products = _repositoryWrapper.SetRepository<Product>().TableNoTracking;
        if (request.IsActive != null)
            products = products.Where(p => p.IsEnable == request.IsActive);
        if (request.ProductName != null)
            products = products.Where(p => p.PersianName.ToLower().Trim().Contains(request.ProductName.ToLower().Trim()));
        if (request.CategoryId != default)
        {
            var cats = await _mediator.Send(new GetProductCategoryChildrenQuery(request.CategoryId), cancellationToken);
            products = products.Where(p => cats.Contains(p.CategoryId));
        }
        if (request.BrandIds is { Length: > 0 })
            products = products.Where(p => request.BrandIds.Contains(p.BrandId));
        if (request.MinPrice > -1)
            products = products.Where(p => p.Cost >= request.MinPrice);
        if (request.MaxPrice > 0)
            products = products.Where(p => p.Cost <= request.MaxPrice);
        if (request.SortBy != QuerySortBy.None)
        {
            products = request.SortBy switch
            {
                QuerySortBy.Cheapest => products.OrderBy(p => p.Cost),
                QuerySortBy.MostExpensive => products.OrderByDescending(p => p.Cost),
                QuerySortBy.MostPopular => products.OrderByDescending(p => p.Rate),
                QuerySortBy.MostViewed => products.OrderByDescending(p => p.Viewed),
                _ => products
            };
        }
        else
        {
            products = products.OrderByDescending(p => p.CreatedAt);
        }

        if (request.SpecialOffer != null)
        {
            if (request.SpecialOffer.Value)
            {
                var productDiscount = await _repositoryWrapper.SetRepository<ProductDiscount>()
                .TableNoTracking
                    .Where(d => d.HasCode == false && d.IsSpecialOffer && d.ExpireDate.Date >= DateTime.Today.Date)
                    .OrderByDescending(d => d.CreatedAt)
                .Select(d => d.ProductId)
                    .ToListAsync(cancellationToken);

                products = products.Where(p => productDiscount.Contains(p.Id));
            }
        }


        List<ProductSDto> productSDtos = await products
            .Skip(request.Page * 20)
            .Take(20)
            .Select(ProductMapper.ProjectToSDto)
            .ToListAsync(cancellationToken);

        foreach (var productSDto in productSDtos)
        {
            await _mediator.Send(new CalculateProductDiscountCommand(productSDto), cancellationToken);
        }

        return productSDtos;
    }
}